define('feature/comments/comment-model', [
    'backbone',
    'backbone-brace',
    'underscore'
], function (
    Backbone,
    Brace,
    _
) {

    function appendQueryParam(qs, name, value) {
        return value == null ? qs : qs + (qs.length ? '&' : '?') + name + '=' + value;
    }

    var Comment = Brace.Model.extend({
        namedAttributes : {
            id : null,
            version : null,
            text : null,
            html : null,
            anchor : null,
            parent : null,
            comments : null,
            avatarSize : null,
            author : null,
            createdDate : null,
            updatedDate : null,
            permittedOperations : null
        },
        validate : function(attributes) {
            if (!attributes.text || !/\S/.test(attributes.text)) {
                return stash_i18n('stash.web.comment.empty', "Please enter some text.");
            }
        },
        url : function() {
            var qs = '';
            qs = appendQueryParam(qs, 'version', this.get('version'));
            qs = appendQueryParam(qs, 'avatarSize', this.get('avatarSize'));
            qs = appendQueryParam(qs, 'markup', true);
            return Brace.Model.prototype.url.apply(this, arguments) + qs;
        },
        forEachCommentInThread : function(fn) {
            fn(this);
            _.each(this.get('comments'), function(comment) {
                comment.forEachCommentInThread(fn);
            });
        },
        sync: function(method, model, options) {
            return Backbone.sync(method, model, _.extend(options, {
                statusCode : {
                    '404' : function(xhr, testStatus, errorThrown, data, fallbackError) {

                        var error = data && data.errors && data.errors.length && data.errors[0];

                        // TODO - our error handling needs some error codes to avoid this kind of heuristic stuff.
                        if (error && error.message && /comment/i.test(error.message)) {
                            //If replying, show a custom error and allow the user to reload the page
                            if (method === 'create' && model.get('parent') != null) {
                                return {
                                    title: stash_i18n('stash.web.comment.notfound', 'Comment not found'),
                                    message: stash_i18n('stash.web.comment.reply.parent.notfound.message', 'The comment you are replying to no longer exists.'),
                                    shouldReload: true,
                                    fallbackUrl: undefined
                                };
                            } else if (method === 'update') {
                                return {
                                    title: stash_i18n('stash.web.comment.notfound', 'Comment not found'),
                                    message: stash_i18n('stash.web.comment.update.notfound.message', 'The comment you are updating no longer exists.'),
                                    shouldReload: true,
                                    fallbackUrl: undefined
                                };
                            }

                        }
                    }
                }
            }));
        }
    });

    // We have to add the type checking after Comment is already created so we can type-check against the Comment class.
    Brace.Mixins.applyMixin(Comment, {
        namedAttributes : {
            comments : [ Comment ]
        }
    });

    return Comment;
});
