define('feature/file-content/binary-view',
    ['jquery', 'underscore', 'util/navbuilder', 'exports', 'eve'],
    function($, _, navbuilder, exports, eve) {

        function getRawUrl(path, revision) {
            return navbuilder
                .currentRepo()
                .browse()
                .path(path.components)
                .at(revision)
                .raw()
                .build();
        }

        function handleBinary(url) {
            var $message = $("<b/>");
            $message.text(stash_i18n('stash.web.diffview.binary.summary', 'This file cannot be rendered in your browser.'));
            var $elem = $("<a />");
            $elem.attr('href', url);
            $elem.text(stash_i18n('stash.web.diffview.link.download', 'Download this file'));
            return {
                $elem : $("<p/>").append($message).append("<br/>").append($elem),
                type : 'link'
            };
        }

        function handleImage(url) {
            var $elem = $("<img />");
            $elem.attr('src', url);
            return {
                $elem : $elem,
                type : 'image'
            };
        }

        var handlerByExtension = {
            png: handleImage,
            jpg: handleImage,
            jpeg: handleImage,
            bmp: handleImage,
            ico: handleImage,
            gif: handleImage,
            svg: handleImage
        };

        function getBinaryHtml(path, revision) {
            var extension = path.extension && path.extension.toLowerCase();
            var url = getRawUrl(path, revision),
                handler = handlerByExtension[extension] || handleBinary;
            return handler(url);
        }

        function renderBinaryFile($container, path, commitRange) {
            var untilRevision = commitRange.getUntilRevision();

            var result = getBinaryHtml(path, untilRevision && untilRevision.getId());
            $container.empty()
                .addClass('binary')
                .append(result.$elem);
            eve('stash.feature.sourceview.onBinary', null, {
                path: path,
                type: result.type,
                revision: untilRevision
            });
            return $container;
        }

        function renderBinaryDiff($container, source, destination, commitRange) {
            var result, sourceType, destinationType,
                untilRevision = commitRange.getUntilRevision(),
                sinceRevision = commitRange.getSinceRevision();

            // add a new toolbar to the diff view, which will be populated with the mode buttons later on.
            $(aui.toolbar2.toolbar2({content: '', extraClasses: 'image-diff-toolbar'})).prependTo($container);

            if (source && sinceRevision) {
                result = getBinaryHtml(source, sinceRevision.getId());
                sourceType = result.type;
                $("<div class='since-revision'>")
                    .addClass('binary')
                    .append($("<h5>").text(stash_i18n('stash.web.diff.since.revision', 'Old')))
                    .append(result.$elem)
                    .appendTo($container);
            }
            if (destination && untilRevision) {
                result = getBinaryHtml(destination, untilRevision.getId());
                destinationType = result.type;
                $("<div class='until-revision'>")
                    .addClass('binary')
                    .append($("<h5>").text(stash_i18n('stash.web.diff.until.revision', 'New')))
                    .append(result.$elem)
                    .appendTo($container);
            }
            eve('stash.feature.diffview.onBinary', this, {
                $container : $container,
                sourcePath : source,
                sourceType : sourceType,
                sinceRevision : sinceRevision,
                destinationPath : destination,
                destinationType : destinationType,
                untilRevision: untilRevision
            });
        }

        // Text files which should be handled as binary
        var binaryExtensions = [
            'svg'
        ];

        function shouldRenderBinary(data) {
            // NB this doesn't actually work for source-view because the data returned is orthogonal to these checks
            // method exists as a single point to refactor/fix later.
            var extension = data.destination && data.destination.extension;
            return data && (data.binary || extension && (_.indexOf(binaryExtensions, extension) >= 0));
        }

        exports.renderBinaryFile = renderBinaryFile;
        exports.renderBinaryDiff = renderBinaryDiff;
        exports.shouldRenderBinary = shouldRenderBinary;
});
