define('feature/file-content', [
    'jquery',
    'eve',
    'memoir',
    'util/event',
    'util/navbuilder',
    'feature/comments',
    'model/file-change-types'
], function(
    $,
    eve,
    memoir,
    eventUtil,
    navbuilder,
    comments,
    ChangeType
) {

        function getRawUrl(path, revisionRef) {
            var urlBuilder = navbuilder
                   .currentRepo()
                   .browse()
                   .path(path);

            if (revisionRef && !revisionRef.isDefault()) {
                urlBuilder = urlBuilder.at(revisionRef.getId());
            }

            return urlBuilder.raw().build();
        }

        function FileContent(containerSelector, id) {

            var self = this;

            this._id = id || undefined;
            this._containerSelector = containerSelector;

            eve.on('stash.feature.filehistory.revisionSelected', function(revision) {
                if (this === self.untilRevisionPicker) {
                    eve('stash.feature.filecontent.untilRevisionChanged', self, revision);
                }
            });

            // inform blame of data loaded by source view
            eve.on('stash.feature.sourceview.dataLoaded', function(start, limit, data) {
                if (this === self.contentView) {
                    self.blame && self.blame.onDataLoaded(start, limit, data);
                }
            });

            // disable blame option and hide blame if the file is binary or empty.
            var onBinaryOrEmptyFile = function(e) {
                if (this === self.contentView && self.blame) {
                    self.blame.setButtonEnabled(false);
                    self.blame.setEnabled(false);
                }
            };
            eve.on('stash.feature.sourceview.onBinary', onBinaryOrEmptyFile);
            eve.on('stash.feature.sourceview.onEmptyFile', onBinaryOrEmptyFile);

            // inform source view if blame is enabled/disabled.
            eve.on('stash.feature.fileblame.enabledStateChanged', function(isEnabled) {
                if (this === self.blame) {
                    self.contentView.includeBlame = isEnabled;
                }
            });

            var enableToolbarButtons = function (enabled) {
                self.blame && self.blame.setButtonEnabled(enabled);
                self.untilRevisionPicker && self.untilRevisionPicker.setButtonEnabled(enabled);
                if (self.$diffButton && self.$diffButton.length) {
                    self.$diffButton
                            .prop('disabled', !enabled)
                            .toggleClass('disabled', !enabled);
                    if (!enabled) {
                        self.$diffButton.removeAttr('href');
                    }
                }
                self.$viewRaw && self.$viewRaw[enabled ? 'fadeIn' : 'fadeOut']('fast');
            };

            eve.on('stash.feature.sourceview.dataLoaded', function(start, limit, data) {
                if (this === self.contentView) {
                    enableToolbarButtons(true);
                }
            });

            eve.on('stash.feature.sourceview.onError', function(errors) {
                if (this === self.contentView) {
                    enableToolbarButtons(false);
                }
            });

            eve.on('stash.feature.diff-view.render.emptydiff', function() {
                // Temp fix for STASHDEV-3177
                self.$self.addClass('empty-diff');
            });
        }

        FileContent.contentType = {
            SOURCE : 'source',
            DIFF : 'diff'
        };

        // These are only implemented in diffs currently, not source.
        FileContent.commentMode = comments.commentMode;

        FileContent.diffPreset = {
            contentType : FileContent.contentType.DIFF,
            blame : false,
            untilRevisionPicker : true,
            rawLink : false,
            sourceLink : false,
            modeToggle : true,
            changeTypeLozenge : false,
            changeModeLozenge : false,
            breadcrumbs : false,
            commentMode : FileContent.commentMode.NONE
        };

        FileContent.sourcePreset = {
            contentType : FileContent.contentType.SOURCE,
            blame : true,
            untilRevisionPicker : true,
            rawLink : true,
            sourceLink : false,
            modeToggle : true,
            changeTypeLozenge : false,
            changeModeLozenge : false,
            breadcrumbs : false,
            commentMode : FileContent.commentMode.NONE
        };

        FileContent.defaults = {
            contentType : FileContent.contentType.SOURCE,
            blame : false,
            untilRevisionPicker : false,
            rawLink : false,
            sourceLink : false,
            modeToggle : false,
            changeTypeLozenge : false,
            changeModeLozenge : false,
            fileIcon : false,
            breadcrumbs : false,
            scrollPaneSelector : undefined,
            commentMode : FileContent.commentMode.REPLY_ONLY,
            pullRequestDiffLink: false
        };

        FileContent.prototype.initToolbarItems = function (headRef, fileChange) {
            var ContentView = this._options.contentType === FileContent.contentType.SOURCE ?
                require('feature/file-content/source-view') :
                require('feature/file-content/diff-view');

            var self = this,
                $container = $(this._containerSelector),
                untilRevision = fileChange.getCommitRange().getUntilRevision(),
                $self = $(stash.feature.fileContent.main($.extend({
                    id: this._id,
                    preloaded : !!fileChange.getDiff(),
                    sourceUrl : this._options.sourceUrl || this._options.modeToggle ? navbuilder
                                    .currentRepo()
                                    .browse()
                                    .path(fileChange.getPath())
                                    .at(headRef.getDisplayId())
                                    .until(untilRevision && untilRevision.getId())
                                    .build() :
                                    '',
                    diffUrl : this._options.modeToggle ? navbuilder
                                .currentRepo()
                                .diff(fileChange)
                                .at(headRef.getDisplayId())
                                .build() :
                                ''
                }, this._options)));

            if (this.contentView) {
                this.contentView.destroy();
            }

            this.$self && this.$self.remove();
            this.$self = $self.appendTo($container);

            var isScrollPane = this._options.scrollPaneSelector === 'self';
            var $contentView = $self.children('.content-view');
            if (isScrollPane) {
                $contentView.addClass('scroll-x');
            }
            this.contentView = new ContentView(
                $contentView,
                isScrollPane ? $self : this._options.scrollPaneSelector,
                this._options);

            var $toolbar = this.$toolbar = $self.children('.source-commands');

            var FileHistory = this._options.untilRevisionPicker ?
                require('feature/file-content/file-history') :
                null;

            var FileBlame = this._options.blame ? require('feature/file-content/file-blame') : null;

            if (this.untilRevisionPicker) {
                this.untilRevisionPicker.destroy();
            }
            if (this._options.untilRevisionPicker) {
                this.untilRevisionPicker = new FileHistory($toolbar.find('.until-changeset-button'), 'until-changeset');
            } else {
                this.untilRevisionPicker = null;
            }

            if (this._options.blame) {
                this.blame = new FileBlame($toolbar.find('.blame-button'), $contentView);
            } else {
                this.blame = null;
            }

            if (this._options.rawLink) {
                this.$viewRaw = $toolbar.find('.raw-view-link');
            } else {
                this.$viewRaw = null;
            }

            if (this._options.breadcrumbs) {
                this.$breadcrumbs = $self.find(".breadcrumbs");
            } else {
                this.$breadcrumbs = null;
            }

            if (this._options.changeTypeLozenge) {
                this.$changeTypeLozenge = $self.find(".change-type-placeholder");
            } else {
                this.$changeTypeLozenge = null;
            }

            if (this._options.changeModeLozenge) {
                this.$changeModeLozenge = $self.find(".change-mode-placeholder");
            } else {
                this.$changeModeLozenge = null;
            }

            if (this._options.sourceLink) {
                this.$viewSource = $self.find(".source-view-link").tooltip({
                    gravity: 'ne'
                });
            } else {
                this.$viewSource = null;
            }

            if (this._options.pullRequestDiffLink) {
                $self.find(".pull-request-diff-link, .pull-request-diff-outdated").tooltip({
                    gravity: 'ne'
                });
            }

            if (this._options.modeToggle) {
                this.$modeToggle = $toolbar.find('.mode-toggle').tooltip({
                    gravity: 'ne'
                });
            } else {
                this.$modeToggle = null;
            }

            this.$sourceButton = $toolbar.find('.mode-source');
            this.$diffButton = $toolbar.find('.mode-diff');
        };

        FileContent.prototype.initForContent = function(headRef, fileChange, lineNumber) {
            var untilRevision = fileChange.getCommitRange().getUntilRevision();

            if (this.blame) {
                this.blame.setEnabled(false); // Currently, when you switch branches, we disable blame if you had it on
                this.blame.init(fileChange.getPath(), untilRevision);
            }

            if (this.$viewSource) {
                if (fileChange.getType() === ChangeType.DELETE) {
                    this.$viewSource.addClass("hidden");
                } else {
                    this.$viewSource.attr('href', navbuilder
                        .currentRepo()
                        .browse()
                        .path(fileChange.getPath())
                        .at(untilRevision && untilRevision.getId())
                        .build());
                }
            }

            if (this.$viewRaw) {
                this.$viewRaw.attr('href', getRawUrl(fileChange.getPath(), untilRevision && untilRevision.getRevisionReference()));
            }

            if (this.untilRevisionPicker) {
                this.untilRevisionPicker.init(fileChange.getPath(), untilRevision && untilRevision.getRevisionReference(), headRef);
            }

            if (this.$breadcrumbs) {
                var components = _.map(fileChange.getPath().getComponents(), function(str) {
                    return { text: str };
                });
                this.$breadcrumbs.html(stash.widget.breadcrumbs.crumbs({pathComponents: components}));
            }

            if (this.$changeTypeLozenge) {

                var srcPath = fileChange.getSrcPath();

                if (fileChange.getType() === ChangeType.RENAME) {
                    srcPath = srcPath.getName();
                } else {
                    srcPath = '/' + srcPath;
                }

                this.$changeTypeLozenge.append(stash.feature.fileContent.fileChangeTypeLozenge({
                    changeType : fileChange.getType(),
                    previousPath : srcPath
                }));
            }

            if (this.$changeModeLozenge) {
                var lozenge = this.getFileChangedModeLozenge(fileChange);
                if (lozenge) {
                    this.$changeModeLozenge.append($(lozenge).tooltip());
                }
            }

            if (this.$modeToggle && memoir.nativeSupport()) {
                this.$modeToggle.on('click', 'a:not(.active,.disabled)', function(e) {
                    if (!eventUtil.openInSameTab(e)) {
                        return;
                    }
                    e.preventDefault();
                    eve('stash.feature.filecontent.requestedModeChange', this, $(this).hasClass('mode-diff') ? FileContent.contentType.DIFF : FileContent.contentType.SOURCE);
                });
            }

            return this.contentView.init(fileChange, {
                targetItem : lineNumber > 0 ? lineNumber - 1 : null,
                lineComments : this._options.lineComments,
                fileComments : this._options.fileComments
            });
        };

        FileContent.prototype.getFileChangedModeLozenge = function(fileChange) {
            var srcExecutable = fileChange.getSrcExecutable(),
                executable = fileChange.getExecutable();

            // executable can be null if the file has been deleted. We want to show the lozenge when a file has been
            // added and is executable, but not when the it has been deleted or when a file has been added without +x
            var added = null;

            if ((srcExecutable == null && executable === true) ||
                (srcExecutable === false && executable === true)) {
                added = true;
            } else if (srcExecutable === true && executable === false) {
                added = false;
            }

            if (added !== null) {
                return $(stash.feature.fileContent.fileChangeModeLozenge({
                    added : added
                }));
            }
            return null;
        };

        FileContent.prototype.init = function(fileChange, optHeadRef, optLineNumber, options) {
            options = this._options = $.extend({}, FileContent.defaults, options);

            var commitRange = fileChange.getCommitRange(),
                headRef = optHeadRef || commitRange.getUntilRevision() && commitRange.getUntilRevision().getRevisionReference();

            if (options.changeTypeLozenge && !fileChange.getType()) {
                throw new Error("Change type is required to show the change type lozenge.");
            }

            if (!commitRange.getUntilRevision() && (options.blame || options.sourceLink || options.rawLink || options.untilRevisionPicker)) {
                throw new Error("Revision info is required to show blame, a link to the source or raw file, or a revision picker.");
            }

            this.initToolbarItems(headRef, fileChange);

            return this.initForContent(headRef, fileChange, optLineNumber);
        };

        /**
         * @param number the line number to select
         * @param optFileIndex the index of the file in which this line resides.  Defaults to 0
         */
        FileContent.prototype.selectLine = function(number, optFileIndex) {
            this.contentView.selectLine(number, optFileIndex || 0);
        };

        FileContent.prototype.destroy = function() {
            if (this.contentView) {
                this.contentView.destroy();
                this.contentView = null; // only destroy it once
            }
        };

        return FileContent;
});
