define('feature/pull-request/pull-request-table', [
    'jquery',
    'aui',
    'eve',
    'model/page-state',
    'widget/paged-table'
], function (
    $,
    AJS,
    eve,
    pageState,
    PagedTable) {

        function PullRequestTable(prState, prOrder, getPullRequestsUrlBuilder, options) {
            this.prState = prState;
            this.prOrder = prOrder === '' ? (prState === 'open' ? 'oldest' : 'newest') : prOrder;
            this.getPullRequestsUrlBuilder = getPullRequestsUrlBuilder;
            this.scope = options && options.scope || 'repository';

            var noneFoundMessage = stash_i18n('stash.web.pull-requests.nonefound', 'No pull requests found.');
            if (prState === 'open' || prState === '') {
                noneFoundMessage = stash_i18n('stash.web.pull-requests.noneopen', 'There are no open pull requests.');
            } else if (prState === 'merged') {
                noneFoundMessage = stash_i18n('stash.web.pull-requests.nonemerged', 'There are no merged pull requests.');
            } else if (prState === 'declined') {
                noneFoundMessage = stash_i18n('stash.web.pull-requests.nonedeclined', 'There are no declined pull requests.');
            }

            noneFoundMessage += ' <br><a href="' + cav_help_url('stash.help.pull.request') + '" target="_blank">' +
                                    stash_i18n('stash.web.pull-requests.more-info', 'Learn about pull requests') +
                                '</a>.';

            var defaults = {
                bufferPixels: 150,
                target: "#pull-requests-table",
                tableMessageClass: "pull-request-table-message",
                noneFoundMessageHtml: noneFoundMessage,
                allFetchedMessageHtml: stash_i18n('stash.web.pull-requests.allfetched', 'No more pull requests')
            };
            options = $.extend({}, defaults, options);

            PagedTable.call(this, options);

            $('.pull-requests-table .author .user-avatar > .aui-avatar-inner > img').tooltip({
                hoverable: false,
                offset: 5,
                delayIn: 0,
                gravity: function () {
                    // Always position on screen
                    return $.fn.tipsy.autoNS.call(this) + $.fn.tipsy.autoWE.call(this);
                },
                live: true
            });
        }

        $.extend(PullRequestTable.prototype, PagedTable.prototype);

        PullRequestTable.prototype.buildUrl = function (start, limit) {
            var self = this;

            return self.getPullRequestsUrlBuilder()
                .withParams({
                    start: start,
                    limit: limit,
                    state: self.prState,
                    order: self.prOrder,
                    avatarSize: stash.widget.avatarSizeInPx({ size: 'medium' })
                }).build();
        };

        PullRequestTable.prototype.focusInitialRow = function () {
            this.$table.find("tbody tr.pull-request-row:first").addClass("focused-pull-request");
        };

        PullRequestTable.prototype.attachNewContent = function (data, attachmentMethod) {
            PagedTable.prototype.attachNewContent.call(this, data, attachmentMethod);

            eve('stash.widget.pullRequestsTable.contentAdded', this, data);
        };

        PullRequestTable.prototype.handleNewRows = function (data, attachmentMethod) {
            var self = this;
            var rows = $(_.map(data.values,function (pr) {
                return stash.feature.pullRequest.pullRequestRow({
                    pullRequest: pr,
                    scope: self.scope,
                    currentUser: pageState.getCurrentUser() && pageState.getCurrentUser().name
                });
            }).join(''));
            this.$table.show().children("tbody")[attachmentMethod !== 'html' ? attachmentMethod : 'append'](rows);
        };

        PullRequestTable.prototype.initShortcuts = function () {
            var self = this,
                sel = this.$table.selector,
                openItemDisabled = false,
                options = {
                    "focusedClass": "focused-pull-request",
                    "wrapAround": false,
                    "escToCancel": false
                },
                $pullRequestFilterTip = $("<span/>").addClass('pull-request-filter-tip').insertAfter(this.$table),
                pullRequestFilterTipTimeout,
                focusedRowSelector = sel + " .pull-request-row." + options.focusedClass,
                rowSelector = focusedRowSelector + ", " +               //Always include the currently selected element, even if it's a filtered row
                    sel + ":not(.filter-current-user, .filter-current-user-unactioned) .pull-request-row, " +   //When not filtering pullrequests, include every row
                    sel + ".filter-current-user .pull-request-row.current-user, " +                               //When filtering pullrequests by user, don't include non-reviewer rows
                    sel + ".filter-current-user-unactioned .pull-request-row.current-user:not(.current-user-actioned)";     //When filtering pullrequests by user unactioned, don't include rows user has approved

            eve.on('stash.keyboard.shortcuts.disableOpenItemHandler', function(){
                openItemDisabled = true;
            });
            eve.on('stash.keyboard.shortcuts.enableOpenItemHandler', function(){
                openItemDisabled = false;
            });

            PullRequestTable.prototype.bindMoveToNextHandler = function(keys) {
                (this.moveToNextItem ? this : AJS.whenIType(keys)).moveToNextItem(rowSelector, options).execute(function() {
                    if ($(rowSelector).last().hasClass(options.focusedClass)) {
                        window.scrollTo(0, document.documentElement.scrollHeight);
                    }
                });
            };

            PullRequestTable.prototype.bindMoveToPreviousHandler = function(keys) {
                (this.moveToPrevItem ? this : AJS.whenIType(keys)).moveToPrevItem(rowSelector, options);
            };

            PullRequestTable.prototype.bindOpenItemHandler = function(keys) {
                (this.execute ? this : AJS.whenIType(keys)).execute(function() {
                    if (!openItemDisabled) {
                        var $focusedItem = AJS.$(focusedRowSelector);
                        if ($focusedItem.length) {
                            window.location.href = $focusedItem.find('td.id a').attr('href');
                        }
                    }
                });
            };

            PullRequestTable.prototype.bindHighlightAssignedHandler = function(keys) {
                (this.execute ? this : AJS.whenIType(keys)).execute(function() {
                    rotateCurrentFilter(self.$table);
                });
            };

            function rotateCurrentFilter($el) {
                var filters = [{
                            name: 'filter-current-user',
                            description: stash_i18n('stash.web.stash.web.pull-requests.filter-current-user', 'Showing your pull requests')
                        },
                        {
                            name: 'filter-current-user-unactioned',
                            description: stash_i18n('stash.web.stash.web.pull-requests.filter-current-user-unactioned', 'Showing your unactioned pull requests')
                        },
                        {
                            name: '',
                            description: stash_i18n('stash.web.stash.web.pull-requests.filter-all', 'Showing all pull requests')
                        }
                    ],
                    lastFilterIndex = filters.length - 1;

                _.any(filters, function(filter, index){
                    if($el.hasClass(filter.name) || index === lastFilterIndex){
                        var newFilter = (index < lastFilterIndex) ? filters[index + 1] : filters[0];

                        $el.removeClass(filter.name).addClass(newFilter.name);
                        showFilterTip(newFilter.description);
                        return true;
                    }
                });
            }

            function showFilterTip(description) {
                $pullRequestFilterTip.text(description).addClass('shown');

                window.clearTimeout(pullRequestFilterTipTimeout);
                pullRequestFilterTipTimeout = window.setTimeout(function(){ $pullRequestFilterTip.removeClass('shown'); }, 1500);
            }

            PagedTable.prototype.initShortcuts.call(this);
        };

        return PullRequestTable;
    });
