define('layout/base/menu/recent-repos', ['jquery', 'underscore', 'util/ajax', 'util/navbuilder', 'util/html', 'exports'],
function($, _, ajax, nav, html, exports) {
    exports.initMenu = function(menuTriggerId) {

        var $menu = $('#' + html.sanitizeId($('#' + menuTriggerId).attr('aria-owns'))).addClass('recent-repositories-menu'),
        $repoList = $menu.find('.recent-repositories-section ul'),
        $loading = $(stash.layout.menu.loadingRecentReposMenuItem());

        $repoList.append($loading);

        var fetchRecentRepos = function() {
            ajax.rest({
                url: nav.rest().profile().recent().repos().build(),
                statusCode : {
                    '*' : function() {
                        return false; // don't show any error messages
                    }
                }
            }).done(function (data) {
                if (data && data.size) {
                    var sortedValues = data.values;
                    sortedValues.sort(function(repo1, repo2) {
                        return repo1.project.name.localeCompare(repo2.project.name) ||
                               repo1.name.localeCompare(repo2.name);
                    });

                    $repoList.append($(stash.layout.menu.recentRepoMenuItems({repos: sortedValues})));
                } else {
                    $repoList.append($(stash.layout.menu.noRecentReposMenuItem()));
                }
            }).fail(function () {
                $repoList.append($(stash.layout.menu.errorLoadingRecentReposMenuItem()));
            }).always(function () {
                $loading.remove();
            });
        };

        //Load only once all other resources have loaded
        $(window).on('load', fetchRecentRepos);
    };
});
