define('model/file-change', [
    'backbone-brace',
    'model/path',
    'model/revision',
    'model/commit-range',
    'model/pull-request',
    'model/conflict',
    'model/file-change-types'
], function (
    Brace,
    Path,
    Revision,
    CommitRange,
    PullRequest,
    Conflict,
    ChangeType
) {

    var FileChange = Brace.Model.extend({
        namedAttributes : {
            "commitRange" : CommitRange,
            "srcPath" : Path,
            "path" : Path,
            "type" : null,
            "conflict" : Conflict,
            "diff" : null,
            "srcExecutable" : null,
            "executable" : null
        },
        initialize : function() {
            this.setType(
                FileChange._mapChangeType(
                    this.getType(),
                    this.getSrcPath(),
                    this.getPath()
                )
            );
        }
    }, {
        fromDiff : function(diffJson, commitRange) {
            return new FileChange({
                commitRange : commitRange,
                srcPath : diffJson.source,
                path : diffJson.destination,
                diff : diffJson
            });
        },
        _mapChangeType : function(modState, srcPath, path) {
            return (modState === ChangeType.MOVE && srcPath && srcPath.isSameDirectory(path)) ?
                    ChangeType.RENAME :
                    ChangeType.changeTypeFromId(modState);
        }
    });

    return FileChange;
});
