define('page/changeset/changeset',
    ['exports', 'jquery', 'eve', 'underscore', 'memoir', 'util/navbuilder', 'model/revision', 'feature/changeset/tree-and-diff-view', 'model/commit-range'],
    function(exports, $, eve, _, memoir, navbuilder, Revision, treeAndDiffView, CommitRange) {
        var ROOT = "ROOT";

        // state data
        var atRevision,
            parentRevisions,
            parentRevisionsById,
            currentParentRevision;

        //DOM elements
        var $diffToTrigger,
            $diffToParentOptions;

        function updateParentSelector(selectedParentRevision) {
            $diffToParentOptions.each(function() {
                var $this = $(this),
                    $thisLink = $this.find('a'),
                    isSelected = $thisLink.attr('data-id') === selectedParentRevision.getId();

                $this.toggleClass('selected', isSelected);

                if (isSelected) {
                    $diffToTrigger.text($this.find('.changesetid').text());
                }
            });
        }

        function initForParentId(parentId) {
            currentParentRevision = Object.prototype.hasOwnProperty.call(parentRevisionsById, parentId) ? parentRevisionsById[parentId] : parentRevisions[0];

            updateParentSelector(currentParentRevision);
        }

        function pushState(newParentId) {
            var newUrl = navbuilder
                        .currentRepo()
                        .changeset(atRevision.getId())
                        .withParams({ to : newParentId })
                        .build();

            memoir.pushState(null, null, newUrl);
        }

        function getParentIdFromUrl(parents) {
            return navbuilder.parse(window.location.href).getQueryParamValue('to') ||
                   (parents.length && parents[0].getId()) ||
                   ROOT;
        }

        function onStateChange() {
            var parentId = getParentIdFromUrl(parentRevisions);

            var parentIdChanged = parentId && parentId !== currentParentRevision.getId();

            if(parentIdChanged) {
                eve.stop();
                // don't propagate the event down to treeAndDiffView, otherwise it will first (incorrectly) make a request for the diff of the current file at the new parent,
                // which is discarded as it is immediately followed by the correct request (diff for first file in the tree at the new revision)
                initForParentId(parentId);
                treeAndDiffView.updateCommitRange(new CommitRange({
                    untilRevision : atRevision,
                    sinceRevision : currentParentRevision
                }));
            }

        }

        function listenForKeyboardShortcutRequests() {
            eve.on('stash.widget.keyboard-shortcuts.register-contexts', function(keyboardShortcuts) {
                keyboardShortcuts.enableContext('changeset');
                keyboardShortcuts.enableContext('diff-tree');
            });

            eve.on('stash.keyboard.shortcuts.requestReturnToCommits', function(keys) {
                (this.execute ? this : AJS.whenIType(keys)).execute(function() {
                    window.location.href = $('#repository-nav-commits').attr('href'); //Make sure we include the sticky branch if there is one
                });
            });
        }

        exports.onReady = function(jsonRevision, jsonParentRevisions, maxChanges, extrasSelector, project, changeset) {
            atRevision = new Revision(jsonRevision);
            parentRevisions = _.map(jsonParentRevisions, function(revisionJson) {
                return new Revision(revisionJson);
            });

            parentRevisionsById = {};

            if (parentRevisions.length) {
                for (var i = 0, l = parentRevisions.length, parent; i < l; i++) {
                    parent = parentRevisions[i];
                    parentRevisionsById[parent.getId()] = parent;
                }
            } else {
                parentRevisionsById[ROOT] = new Revision({ id : ROOT });
            }

            var $diffToToolbar = $('.changeset-metadata-diff-to');

            $diffToParentOptions = $diffToToolbar.find('.aui-dropdown2 .changeset-list-item');

            $diffToTrigger = $diffToToolbar.find('.dropdown-text');

            if (memoir.nativeSupport()) {
                $diffToParentOptions.click(function(e) {
                    e.preventDefault();

                    var $newParent = $(this),
                        newParentId = $newParent.find('a').attr('data-id');
                    $diffToParentOptions.removeClass('selected').addClass('unselected');

                    $newParent.addClass('selected').removeClass('unselected');
                    if (newParentId !== currentParentRevision.getId()) {
                        pushState(newParentId);
                    }

                });
            }

            // memoir fires a changestate event when the hash changes for browsers that support pushState...
            eve.on('memoir.changestate', onStateChange);

            // ...but for browsers that don't support pushState, we have to manually listen for it.
            if (!memoir.nativeSupport()) {
                $(window).hashchange(onStateChange);
            }

            initForParentId(getParentIdFromUrl(parentRevisions));

            treeAndDiffView.init(new CommitRange({
                untilRevision: atRevision,
                sinceRevision : currentParentRevision
            }), {
                maxChanges : maxChanges,
                numberOfParents : parentRevisions.length
            });

            listenForKeyboardShortcutRequests();

            $(extrasSelector).append(stash.page.changesetExtrasWebSectionsAndPanels({
                project: project,
                changeset: changeset
            }));
        };
    });
