define('page/filebrowser', ['jquery', 'aui', 'eve', 'memoir', 'feature/filebrowser/fileTable', 'feature/filebrowser/file-finder', 'model/revision-reference', 'model/path', 'exports'],
    function($, AJS, eve, memoir, filetable, finder, RevisionReference, Path, exports) {

    var FileTable = filetable.FileTable,
        FileTableView = filetable.FileTableView,
        FileFinder = finder.FileFinder,
        dialogIsShowing;

    var fileTable,
        fileTableView,
        fileFinder;

    var findFilesTooltip,
        browseFilesTooltip,
        $findFilesItem,
        $content,
        $findFilesLink;

    function getFileNamesFromDOM() {
        return $content.find(".file-row a").map(function (i, row) {
            var $row = $(row);
            return {name: $row.text(), contentId: $row.attr('data-contentId'), type: $row.parent().parent().hasClass('file') ? 'FILE' : 'DIRECTORY'};
        });
    }

    exports.onReady = function(path, revisionRef, fileTableContainer) {

        var currentPath = new Path(path),
            currentRevisionRef = new RevisionReference(revisionRef);

        $findFilesItem = $('.find-files');
        $content = $('.filebrowser-content');
        $findFilesLink = $findFilesItem.children('a');

        fileTableView = new FileTableView(fileTableContainer);

        fileTable = new FileTable(currentPath, currentRevisionRef);

        fileFinder = new FileFinder(fileTableContainer, currentRevisionRef);

        $(document).on('focus', '#browse-table tr.file-row', function() {
            $('.focused-file').removeClass('focused-file');
            $(this).addClass('focused-file');
        });

        eve.on('memoir.popstate', function(e) {
            if (e.state) {
                currentRevisionRef = new RevisionReference(e.state.revisionRef);
            } // else ignore hashchange events
        });

        var pipeRevisionChanged = function(revisionReference) {
            if (currentRevisionRef !== revisionReference) {
                currentRevisionRef = revisionReference;
                eve('stash.page.filebrowser.revisionRefChanged', null, revisionReference);
            }
        };
        eve.on('stash.layout.branch.revisionRefChanged', pipeRevisionChanged);
        eve.on('stash.feature.filetable.revisionRefChanged', pipeRevisionChanged);

        eve.on('stash.widget.branchselector.dialogShown', function () {
            dialogIsShowing = true;
        });
        eve.on('stash.widget.branchselector.dialogHidden', function () {
            dialogIsShowing = false;
        });

        var pipeUrlChanged = function(url) {
            eve('stash.page.filebrowser.urlChanged', null, url);
        };
        eve.on('stash.layout.*.urlChanged', pipeUrlChanged);
        eve.on('stash.feature.*.urlChanged', pipeUrlChanged);

        eve.on('stash.feature.*.pathChanged', function(path) {
            currentPath = path;
            eve('stash.page.filebrowser.pathChanged', null, currentPath);
        });

        eve.on('stash.widget.keyboard-shortcuts.register-contexts', function(keyboardShortcuts) {
            keyboardShortcuts.enableContext('filebrowser');
        });

        var showFinder = function () {
            $findFilesLink
                .text(stash_i18n('stash.web.filefinder.browsefiles', 'Browse files'))
                .attr('title', browseFilesTooltip);
            $findFilesItem.addClass('browse').removeClass('search');
            fileFinder.loadFinder();
        };

        var hideFileFinderAndBrowseFilesLink = function() {
            if (fileFinder.isLoaded()) {
                $findFilesLink
                    .text(stash_i18n('stash.web.filefinder.findfiles', 'Find files'))
                    .attr('title', findFilesTooltip);
                $findFilesItem.addClass('search').removeClass('browse');
                fileFinder.unloadFinder();
            }
        };

        var hideFinder = function() {
            if (fileFinder.isLoaded()) {
                hideFileFinderAndBrowseFilesLink();

                if (fileTable.data) {
                    fileTableView.update(fileTable.data);
                }
                else {
                    fileTable.reload();
                }
            }
        };

        eve.on('stash.feature.filetable.showFind', showFinder);
        eve.on('stash.feature.filetable.hideFind', hideFinder);
        //If the revision has changed, close the file finder and show the browse file link
        eve.on('stash.page.filebrowser.revisionRefChanged', hideFileFinderAndBrowseFilesLink);

        /**
         * Notify listeners of the initial files and when there are subsequent changes.
         */
        eve('stash.feature.filebrowser.filesChanged', null, {
            files:getFileNamesFromDOM(),
            path:new Path(path),
            // TODO There isn't a good way to get the qualified branch/tag at the moment
            revision: currentRevisionRef.getDisplayId()
        });
        eve.on('stash.feature.filetable.dataReceived', function (data) {
            eve('stash.feature.filebrowser.filesChanged', null, {files:data.children.values, path:data.path, revision:data.revision});
        });

        $findFilesLink.click(function () {
            fileFinder.isLoaded() ? hideFinder() : showFinder();
            return false;
        });

        listenForKeyboardShortcutRequests();
    };

    function listenForKeyboardShortcutRequests() {
        var options = {
                "focusedClass": "focused-file",
                "wrapAround": false,
                "escToCancel": false
        },
        rowSelector = "#browse-table tr.file-row",
        focusedRowSelector = rowSelector + '.' + options.focusedClass;

        eve.on('stash.keyboard.shortcuts.requestMoveToNextHandler', function(keys) {
            (this.moveToNextItem ? this : AJS.whenIType(keys)).moveToNextItem(rowSelector, options);
        });
        eve.on('stash.keyboard.shortcuts.requestMoveToPreviousHandler', function(keys) {
            (this.moveToPrevItem ? this : AJS.whenIType(keys)).moveToPrevItem(rowSelector, options);
        });
        eve.on('stash.keyboard.shortcuts.requestOpenItemHandler', function(keys) {
            (this.execute ? this : AJS.whenIType(keys)).execute(function() {
                if (!dialogIsShowing) {
                    var $focusItem = $(focusedRowSelector);
                    if ($focusItem.length) {
                        if ($focusItem.hasClass('file') || !memoir.nativeSupport()) {
                            eve('stash.feature.filetable.showSpinner', this);
                            window.location.href = $focusItem.find('a').attr('href');
                        } else {
                            $focusItem.find('a').click();
                        }
                    }
                }
            });
        });
        eve.on('stash.keyboard.shortcuts.requestOpenParentHandler', function(keys) {
            (this.execute ? this : AJS.whenIType(keys)).execute(function() {
                if (!dialogIsShowing) {
                    var $parentDir = $(fileTableView.getParentDirSelector());
                    if ($parentDir.length) {
                        if (memoir.nativeSupport()) {
                            $parentDir.click();
                        } else {
                            eve('stash.feature.filetable.showSpinner', this);
                            window.location.href = $parentDir.attr('href');
                        }
                    }
                }
            });
        });

        eve.on('stash.keyboard.shortcuts.requestOpenFileFinderHandler', function (keys) {
            findFilesTooltip = stash_i18n('stash.web.filefinder.findfiles.tooltip',
                "Find files in this repository (Type ''{0}'')", keys);
            $findFilesLink.attr('title', findFilesTooltip);
            (this.execute ? this : AJS.whenIType(keys)).execute(function() {
                eve('stash.feature.filetable.showFind', this);
            });
        });
        eve.on('stash.keyboard.shortcuts.requestCloseFileFinderHandler', function (keys) {
            browseFilesTooltip = stash_i18n('stash.web.filefinder.browse.files.tooltip',
                "Browse files in this repository (Type ''{0}'')", keys);
            (this.execute ? this : AJS.whenIType(keys)).execute(function() {
                eve('stash.feature.filetable.hideFind', this);
            });
        });
    }
});
