define('util/event', ['jquery', 'eve', 'underscore', 'util/navigator',  'exports'],
    function($, eve, _, navigator, exports) {

    /**
     * Returns true if a mouse click event should be handled in the same tab, false otherwise
     * @param e a jquery mouse event
     */
    exports.openInSameTab = function (e) {
        return (!e.which || e.which === 1) &&
              !(e.metaKey || e.ctrlKey || e.shiftKey || (e.altKey && !$.browser.msie));
    };



    /**
     * Linux: any modifier prevents scroll
     * FF: prevent scroll in Win and Mac when ANY modifier is pressed.
     * Chrome : Alt on Windows, Cmd on Mac handle history nav, and Shift on both OSes handles text highlighting.
     * Safari: Alt is history nav in Windows, Ctrl and Cmd both do it on Mac, and Shift only avoids scroll on Mac.
     * IE8-9: Alt key is history navigation.
     * @param e key event
     */
    exports.modifiersPreventScroll = function(e) {
        if ($.browser.mozilla || (/Linux/).test(navigator._getPlatform())) {

            return  e.altKey || e.shiftKey || e.ctrlKey || e.metaKey || false;

        } else if ($.browser.webkit && (/Chrome/).test(navigator._getUserAgent())) {

            return (/Win/).test(navigator._getPlatform()) ?
                e.altKey  || e.shiftKey || false :
                e.metaKey || e.shiftKey || false;

        } else if ($.browser.safari) {

            return (/Win/).test(navigator._getPlatform()) ?
                e.altKey || false :
                e.altKey || e.shiftKey || e.ctrlKey || e.metaKey || false;

        } else if ($.browser.msie) {

            return e.altKey || false;

        } else {
            return false;
        }
    };


    /**
     * When enabled, this function will send an event when a user changes their font size.
     */
    exports.listenForFontSizeChange = _.once(function() {
        var heightTest = $('<div style="position: fixed; visibility: hidden; speak: none; height: auto; top: -999px; left: -999px;">Ignore this text</div>').appendTo(document.body),
            heightTestHeight = heightTest.height(),
            checkHeight,
            interval = 500;

        setTimeout(checkHeight = function() {
            var newHeight = heightTest.height();
            if (newHeight !== heightTestHeight) {
                heightTestHeight = newHeight;
                eve('stash.util.events.fontSizeChanged');
            }
            setTimeout(checkHeight, interval);
        }, interval);
    });
});
