define('widget/approve', [
    'jquery',
    'eve',
    'util/ajax',
    'model/page-state',
    'widget/button-trigger'
], function(
    $,
    eve,
    ajax,
    pageState,
    ButtonTrigger
) {

    function Approve(selectorTrigger, url) {
        this._opts = {
            url: url,
            triggerHandler: this.buttonClicked
        };

        ButtonTrigger.call(this, selectorTrigger, this._opts);

        var self = this;

        var handler = function (data) {
            if (pageState.getCurrentUser() && data.user.name === pageState.getCurrentUser().name &&
                data.pullRequestId === pageState.getPullRequest().getId()) {
                self._$trigger.find('.trigger-text')
                    .attr('title', data.approved ?
                        stash_i18n("stash.web.pull-request.toolbar.approved.tooltip", "You have approved this pull request") :
                        stash_i18n("stash.web.pull-request.toolbar.approved.tooltip", "Approve this pull request"));
                self.setTriggerActive(data.approved);
            }
        };

        // optimistic event listening (assume it works, then revert if necessary)
        eve.on('stash.widget.approve-button.adding', handler);
        eve.on('stash.widget.approve-button.removing', handler);
        eve.on('stash.widget.approve-button.add.failed', handler);
        eve.on('stash.widget.approve-button.remove.failed', handler);
    }
    $.extend(Approve.prototype, ButtonTrigger.prototype);

    Approve.prototype.buttonClicked = function (isOn, event) {
        var self = this;

        function fireEvent(eventName, approved) {
            eve(eventName, self, {
                approved : approved,
                pullRequestId: pageState.getPullRequest().getId(),
                user: pageState.getCurrentUser()
            });
        }

        fireEvent(isOn ? 'stash.widget.approve-button.adding' : 'stash.widget.approve-button.removing', isOn);

        ajax.rest({
            url: self._opts.url,
            type: isOn ? 'POST' : 'DELETE',
            statusCode: {
                '401' : function(xhr, textStatus, errorThrown, errors, dominantError) {
                    return $.extend({}, dominantError, {
                        title: stash_i18n("stash.web.pull-request.approve.error.401.title", "Could not approve pull request."),
                        message: stash_i18n("stash.web.pull-request.approve.error.401.message", "You do not have permission to approve this pull request."),
                        fallbackUrl: false,
                        shouldReload: true
                    });
                },
                '409' : function(xhr, textStatus, errorThrown, errors, dominantError) {
                    return $.extend({}, dominantError, {
                        title: stash_i18n("stash.web.pull-request.approve.error.409.title", "Could not approve pull request."),
                        fallbackUrl: false,
                        shouldReload: true
                    });
                }
            }
        }).done(function() {
            fireEvent(isOn ? 'stash.widget.approve-button.added' : 'stash.widget.approve-button.removed', isOn);
        }).fail(function () {
            fireEvent(isOn ? 'stash.widget.approve-button.add.failed' : 'stash.widget.approve-button.remove.failed', !isOn);
        });
    };

    return Approve;
});
