define('widget/pager-filter', ['jquery', 'underscore', 'util/ajax', 'widget/submit-spinner'], function($, _, ajax, SubmitSpinner) {

    function PageFilter(filter, pagers, templates, throttleMs) {
        this.$filter = $(filter);
        this.pager = this._matchTarget(pagers);
        this.templates = templates;
        this.throttleMs = throttleMs ? throttleMs : 350;
        this.filterSpinner = new SubmitSpinner(this.$filter, 'after');
        this._bind();
    }

    PageFilter.prototype.getTarget = function() {
        return this.$filter.data('for');
    };

    PageFilter.prototype._matchTarget = function (pagersByTarget) {
        var self = this;
        var match;
        _.each(pagersByTarget, function(pagers, target) {
            if (target == self.getTarget()) {
                match = pagers[0]; // just need one, it will update its siblings
            }
        });
        return match;
    };

    PageFilter.prototype.reload = function () {
        var self = this;
        self.$filter.attr("data-loading", "true");
        self.filterSpinner.show();
        var filter = self.$filter.val();
        return ajax.rest({
                url: self.$filter.data('url'),
                data: {
                    filter: filter
                }
            }).done(function (page) {
                // if there is a pager, delegate it to refresh the page and update its siblings, otherwise do it manually
                if (self.pager) {
                    self.pager.refresh(page);
                } else {
                    var target = self.getTarget();
                    var template = self.templates[target];
                    //HACK: Workaround for STASHDEV-3225
                    page.filter = filter;
                    $("#" + target).html(template(page));
                }
                self.$filter.attr("data-loading", "false");
            }).always(function () {
                self.filterSpinner.hide();
            });
    };
    
    PageFilter.prototype._bind = function () {
        var self = this;
        this.$filter.keyup(_.debounce(function () {
            self.reload();
        }, this.throttleMs));
    };

    var pageFilter;
    
    PageFilter.onReady = function(filter, pagers, templates, throttleMs) {
        pageFilter = new PageFilter(filter, pagers, templates, throttleMs);
        return pageFilter;
    };
    
    PageFilter.reload = function() {
        return pageFilter && pageFilter.reload();
    };

    return PageFilter;

});


